unit DataWin;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Db, Grids, DBGrids, StdCtrls, DBCtrls, DBTables, Menus, MultFilt,
  IniFiles;

const
   // Filter Panel Constants
   COLWIDTH        = 150;
   HMARGIN         = 8;
   VMARGIN         = 4;
   MAX_COMBO_ITEMS = 64;
   FCNAME          = 'Filter_Con_';

type
   TFilterCombo = class(TComboBox)
   protected
      MultForm    : TMultFiltForm;
   public
      Multiples   : boolean;
      DataField   : TField;      // The LOOKUP field this combobox refers to
      function    ShowMultForm: integer;
   end;

   TBuddyLabel = class(TLabel)
   public
      Buddy       : TControl;
   end;

  TDataWinForm = class(TForm)
    Panel1: TPanel;
    DBNavigator1: TDBNavigator;
    Label1: TLabel;
    IndexCombo: TComboBox;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    FilterBut: TButton;
    FilterPanel: TPanel;
    TablePopupMenu: TPopupMenu;
    ExporttoExcel1: TMenuItem;
    FastExcel971: TMenuItem;
    SlowExcel951: TMenuItem;
    ExporttoAscii1: TMenuItem;
    ImportTablefromAscii1: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    RefreshTable1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    ViewTableStructure1: TMenuItem;
    EmptyTable1: TMenuItem;
    StatusPanel: TPanel;
    StatPanel1: TPanel;
    StatPanel2: TPanel;
    ViewStatusBar1: TMenuItem;
    RefreshFilterCombos1: TMenuItem;
    Font1: TMenuItem;
    FontDialog1: TFontDialog;
    Exclusive1: TMenuItem;
    Admin1: TMenuItem;
    CreateIndex1: TMenuItem;
    DeleteIndex1: TMenuItem;
    N4: TMenuItem;
    LookupFilterPopup: TPopupMenu;
    MultFilt1: TMenuItem;
    ResetTableViewtoDefault1: TMenuItem;
    NumFilterPopup: TPopupMenu;
    FilteronRange1: TMenuItem;
    procedure IndexComboChange(Sender: TObject);
    procedure DBGrid1TitleClick(Column: TColumn);
    procedure FilterButClick(Sender: TObject);
    procedure FilterPanelResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure RefreshFilterCombos1Click(Sender: TObject);
    procedure FastExcel971Click(Sender: TObject);
    procedure SlowExcel951Click(Sender: TObject);
    procedure ExporttoAscii1Click(Sender: TObject);
    procedure RefreshTable1Click(Sender: TObject);
    procedure ImportTablefromAscii1Click(Sender: TObject);
    procedure ViewTableStructure1Click(Sender: TObject);
    procedure EmptyTable1Click(Sender: TObject);
    procedure TablePopupMenuPopup(Sender: TObject);
    procedure ViewStatusBar1Click(Sender: TObject);
    procedure Font1Click(Sender: TObject);
    procedure Exclusive1Click(Sender: TObject);
    procedure CreateIndex1Click(Sender: TObject);
    procedure DeleteIndex1Click(Sender: TObject);
    procedure MultFilt1Click(Sender: TObject);
    procedure ResetTableViewtoDefault1Click(Sender: TObject);
    procedure DBGrid1ColumnMoved(Sender: TObject; FromIndex,
      ToIndex: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FilteronRange1Click(Sender: TObject);
  protected
    NumFilter  : integer;
    procedure  InitIndexCombo;
    procedure  InitFilterControls;
    procedure  LayoutFilterPanel;
    procedure  Fill_Filter_Combo(cb: TFilterCombo);
    procedure  TableFilterRecord(DataSet: TDataSet; var Accept: Boolean);
    procedure  FilterComboChange(Sender: TObject);
    procedure  FilterLabelClick(Sender: TObject);
    procedure  DateFilterLabelClick(Sender: TObject);
    procedure  NumLabelClick(Sender: TObject);
    procedure  NumKeyPress(Sender: TObject; var Key: char);
    procedure  UpdateNumRecords;
    procedure  FormHint(Sender: TObject);
    function   SetExclusiveMode(Mode: boolean): boolean;
    procedure  AddLookupFilter(f: TField; FilterNum: integer);
    procedure  AddDateFilter(f: TField; FilterNum: integer);
    procedure  AddNumFilter(f: TField; FilterNum: integer);
  public
    CurTable            : TTable;
    IgnoreTitleClick    : boolean;
    DefColList          : TStringList;
    procedure  AssignTable(Table: TTable);
    procedure  Refresh_Filter_Combos;
    function   GetFilterDesc: TStringList;
    procedure  SaveColumnSettings;
    procedure  RestoreColumnSettings;
    function   GetColSetName: string;
    procedure  SaveDefaultColumnSettings;
  end;

implementation

{$R *.DFM}

uses TableUtils, TableStructure, CreateIndexFrm, DateFilter, NumFilter;

procedure TDataWinForm.AssignTable(Table: TTable);
begin
   Assert(Table<>nil);
   SaveColumnSettings;

   CurTable := Table;
   DataSource1.DataSet := CurTable;

   SaveDefaultColumnSettings;

   if CurTable<>nil then
      RestoreColumnSettings;

   CurTable.Filtered := True;
   CurTable.OnFilterRecord := TableFilterRecord;

   InitIndexCombo;
   InitFilterControls;
   UpdateNumRecords;
end;

procedure TDataWinForm.InitIndexCombo;
var
   i : integer;
begin
   IndexCombo.Items.Clear;
   CurTable.IndexDefs.Update;
   for i := 0 to CurTable.IndexDefs.Count-1 do begin
      IndexCombo.Items.Add(CurTable.IndexDefs[i].Fields);
   end;
   IndexCombo.ItemIndex := IndexCombo.Items.IndexOf(CurTable.IndexFieldNames);
end;

procedure TDataWinForm.IndexComboChange(Sender: TObject);
var
   i : integer;
begin
   i := IndexCombo.ItemIndex;
   if i = -1 then exit;
   CurTable.IndexFieldNames := IndexCombo.Items[i];
end;

procedure TDataWinForm.DBGrid1ColumnMoved(Sender: TObject; FromIndex,
  ToIndex: Integer);
begin
   IgnoreTitleClick := True;
end;

procedure TDataWinForm.DBGrid1TitleClick(Column: TColumn);
var
   f : TField;
   s : string;
   i : integer;
begin
   if IgnoreTitleClick then begin
      // This is used to avoid a fake click when the user just wants
      // to move a column.
      IgnoreTitleClick := False;
      exit;
   end;

   f := Column.Field;
   s := f.FieldName;

   if f.FieldKind = fkLookup then begin
      s := f.KeyFields;
   end;

   CurTable.IndexFieldNames := s;

   for i := 0 to IndexCombo.Items.Count-1 do begin
      if Copy(IndexCombo.Items[i], 1, Length(s)) = s then
         IndexCombo.ItemIndex := i;
   end;
end;

procedure TDataWinForm.FilterButClick(Sender: TObject);
begin
   FilterPanel.Visible := not FilterPanel.Visible;

   if FilterPanel.Visible then
      FilterBut.Caption := '&Hide Filters'
   else
      FilterBut.Caption := '&Show Filters'
end;

procedure TDataWinForm.Fill_Filter_Combo(cb: TFilterCombo);
var
   ds          : TDataSet;
   f           : TField;
   IDFldName   : string;
   TextFldName : string;
   Num         : integer;
begin
   cb.Items.Clear;
   f  := cb.DataField;
   ds := f.LookupDataSet;
   IDFldName   := f.LookupKeyFields;
   TextFldName := f.LookupResultField;

   if ds.Active=False then exit;

   ds.First;
   Num := 0;
   while not ds.EOF do begin
      Num := Num + 1;
      if Num > MAX_COMBO_ITEMS then break;
      
      cb.Items.AddObject(ds.FieldByName(TextFldName).AsString,
         pointer(ds.FieldByName(IDFldName).AsInteger));
      ds.Next;
   end;

   cb.Multiples := False;
   if cb.MultForm<>nil then
      cb.MultForm.ListBox1.Items.Assign(cb.Items);
end;

procedure TDataWinForm.AddNumFilter(f: TField; FilterNum: integer);
var
   lb       : TLabel;
   ne       : TNumFilter;
begin
   lb := TLabel.Create(FilterPanel);
   lb.Name           := 'Filter_Label_'+IntToStr(FilterNum);
   lb.Parent         := FilterPanel;
   lb.Caption        := f.DisplayName;
   lb.Hint           := 'Double click to clear this filter';
   lb.OnDblClick     := NumLabelClick;

   ne := TNumFilter.Create(FilterPanel);
   ne.Name           := FCNAME+IntToStr(FilterNum);
   ne.Parent         := FilterPanel;
   ne.Width          := COLWIDTH - HMARGIN;
   ne.DataField      := f;
   ne.Hint           := 'Enter a number to filter on (and press Enter), or right click for a filter range';
   ne.OnKeyPress     := NumKeyPress;
   ne.AutoSelect     := True;
   ne.PopupMenu      := NumFilterPopup;
   ne.UpdateText;

   lb.FocusControl := ne;
end;

procedure TDataWinForm.AddLookupFilter(f: TField; FilterNum: integer);
var
   lb       : TLabel;
   cb       : TFilterCombo;
begin
   lb := TLabel.Create(FilterPanel);
   lb.Name           := 'Filter_Label_'+IntToStr(FilterNum);
   lb.Parent         := FilterPanel;
   lb.Caption        := f.DisplayName;
   lb.Hint           := 'Double click to clear this filter';
   lb.OnDblClick     := FilterLabelClick;

   cb := TFilterCombo.Create(FilterPanel);
   cb.Name           := FCNAME+IntToStr(FilterNum);
   cb.Parent         := FilterPanel;
   cb.Width          := COLWIDTH - HMARGIN;
   cb.Style          := csDropDownList;
   cb.DropDownCount  := 12;
   cb.DataField      := f;
   cb.OnChange       := FilterComboChange;
   cb.Hint           := 'Select an item to Filter the table, right click for Multiple filters';
   cb.PopupMenu      := LookupFilterPopup;
   Fill_Filter_Combo(cb);

   lb.FocusControl := cb;
end;

procedure TDataWinForm.AddDateFilter(f: TField; FilterNum: integer);
var
   lb       : TBuddyLabel;
   fc       : TFilterDate;
begin
   lb := TBuddyLabel.Create(FilterPanel);
   lb.Name           := 'Filter_Label_'+IntToStr(FilterNum);
   lb.Parent         := FilterPanel;
   lb.Caption        := f.DisplayName;
   lb.Hint           := 'Double click to clear this filter';
   lb.OnDblClick     := DateFilterLabelClick;

   fc := TFilterDate.Create(FilterPanel);
   fc.Name           := FCNAME+IntToStr(FilterNum);
   fc.Parent         := FilterPanel;
   fc.Width          := COLWIDTH - HMARGIN;
   fc.Height         := 21;
   fc.Alignment      := taLeftJustify;
   fc.Hint           := 'Double click to change this filter';
   fc.DataField      := f;
   fc.OnDblClick     := DateFilterLabelClick;
   fc.MinDate        := Now;
   fc.MaxDate        := Now;
   fc.UpdateCaption;

   lb.Buddy := fc;
end;

procedure TDataWinForm.InitFilterControls;
var
   i, j     : integer;
   f        : TField;
begin
   FilterPanel.Visible := False;

   // Clear out any old controls
   for i := FilterPanel.ControlCount-1 downto 0 do begin
      FilterPanel.Controls[i].Free;
   end;

   j := 0;
   for i := 0 to CurTable.FieldCount-1 do begin
      f := CurTable.Fields[i];
      if not f.Visible then continue;

      if f.FieldKind = fkLookup then begin
         AddLookupFilter(f, j);
         j := j + 1;
      end;

      if f.DataType = ftDateTime then begin
         AddDateFilter(f, j);
         j := j + 1;
      end;

      if (f.DataType = ftInteger) or (f.DataType = ftFloat) then begin
         AddNumFilter(f, j);
         j := j + 1;
      end;

   end;
   NumFilter := j;

   if NumFilter = 0 then begin
      FilterPanel.Visible := False;
      FilterBut.Visible := False;
   end else begin
      FilterPanel.Visible := True;
      FilterBut.Visible := True;
      FilterBut.Enabled := True;
      FilterBut.Caption := '&Hide Filters';
      LayoutFilterPanel;
   end;
end;

procedure TDataWinForm.LayoutFilterPanel;
var
   i        : integer;
   lb       : TControl;
   cb       : TControl;
   CurTop   : integer;
   CurLeft  : integer;
   MaxTop   : integer;
begin
   CurTop  := VMARGIN;
   CurLeft := HMARGIN;
   MaxTop  := CurTop;
   for i := 0 to NumFilter-1 do begin
      lb := FilterPanel.FindComponent('Filter_Label_'+IntToStr(i)) as TControl;
      lb.Top      := CurTop;
      lb.Left     := CurLeft;

      cb := FilterPanel.FindComponent(FCNAME+IntToStr(i)) as TControl;
      cb.Top      := CurTop + lb.Height;
      cb.Left     := CurLeft;

      MaxTop  := CurTop + lb.Height + cb.Height;

      CurLeft := CurLeft + COLWIDTH;
      if CurLeft + COLWIDTH > FilterPanel.Width then begin
         CurLeft := HMARGIN;
         CurTop := CurTop + lb.Height + cb.Height + VMARGIN;
      end;
   end;

   FilterPanel.Height := MaxTop + VMARGIN;
end;

procedure TDataWinForm.Refresh_Filter_Combos;
var
   i  : integer;
   fc : TControl;
   cb : TFilterCombo;
begin
   for i := 0 to NumFilter-1 do begin
      fc := FilterPanel.FindComponent(FCNAME+IntToStr(i)) as TControl;
      if fc is TFilterCombo then begin
         cb := fc as TFilterCombo;
         Fill_Filter_Combo(cb);
      end;
   end;
end;

procedure TDataWinForm.FilterPanelResize(Sender: TObject);
begin
   LayoutFilterPanel;
end;

procedure TDataWinForm.FormCreate(Sender: TObject);
begin
   NumFilter := 0;
   Application.OnHint := FormHint;
   DefColList := TStringList.Create;
end;

procedure TDataWinForm.FormDestroy(Sender: TObject);
var
   i : integer;
begin
   for i := 0 to DefColList.Count-1 do
      DefColList.Objects[i].Free;
   DefColList.Free;
end;


procedure TDataWinForm.RefreshFilterCombos1Click(Sender: TObject);
begin
   Refresh_Filter_Combos;
end;

procedure TDataWinForm.UpdateNumRecords;
begin
   StatPanel1.Caption := Format('Num Records = %d', [CurTable.RecordCount]);
end;

procedure TDataWinForm.FilterComboChange(Sender: TObject);
var
   cb    : TFilterCombo;
   idx   : integer;
begin
   cb := Sender as TFilterCombo;
   cb.Multiples := False;
   idx := cb.ItemIndex;
   cb.Style := csDropDownList;
   cb.ItemIndex := idx;

   CurTable.Refresh;
   UpdateNumRecords;
end;

procedure TDataWinForm.DateFilterLabelClick(Sender: TObject);
var
   df : TDateFilterForm;
   fd : TFilterDate;
begin
   if (Sender is TBuddyLabel) then begin
      fd := (Sender as TBuddyLabel).Buddy as TFilterDate;
      fd.UseMin := False;
      fd.UseMax := False;
      fd.IncBlanks := True;
      fd.UpdateCaption;
   end;

   if (Sender is TFilterDate) then begin
      df := TDateFilterForm.Create(Self);
      try
         df.fd := (Sender as TFilterDate);
         df.ShowModal;
      except
         on e: Exception do begin
            e.Message := 'User does not have latest Common Controls DLL?' +
               #13#10#13#10 + e.Message;
            df.Free;
            raise;
         end;
      end;
   end;

   CurTable.Refresh;
   UpdateNumRecords;
end;

procedure TDataWinForm.FilterLabelClick(Sender: TObject);
var
   cb : TFilterCombo;
begin
   cb := (Sender as TLabel).FocusControl as TFilterCombo;
   cb.ItemIndex := -1;
   cb.OnChange(cb);
end;

procedure TDataWinForm.TableFilterRecord(DataSet: TDataSet; var Accept: Boolean);
var
   i, j     : integer;
   idx      : integer;
   FilterID : integer;
   fc       : TControl;
   cb       : TFilterCombo;
   fd       : TFilterDate;
   ne       : TNumFilter;
   f, f2    : TField;
   MatchOne : boolean;
begin
   Accept := True;

   for i := 0 to NumFilter-1 do begin
      fc := FilterPanel.FindComponent(FCNAME+IntToStr(i)) as TControl;

      if fc is TFilterDate then begin
         fd := fc as TFilterDate;

         if not fd.IsDateInFilter(fd.DataField.AsDateTime) then begin
            Accept := False;
            exit;
         end;
      end;

      if fc is TNumFilter then begin
         ne := fc as TNumFilter;

         if not ne.IsNumInFilter(ne.DataField.AsFloat) then begin
            Accept := False;
            exit;
         end;
      end;

      if fc is TFilterCombo then begin
         cb := fc as TFilterCombo;

         f := cb.DataField;
         f2 := f.DataSet.FieldByname(f.KeyFields);

         if cb.Multiples then begin
            // We have a multiple-selection filter
            MatchOne := False;
            for j := 0 to cb.Items.Count-1 do begin
               FilterID := integer(cb.Items.Objects[j]);
               if cb.MultForm.ListBox1.Selected[j] and (FilterID = f2.AsInteger) then begin
                  MatchOne := True;
                  break;
               end;
            end;
            if not MatchOne then begin
               Accept := False;
               exit;
            end;
         end else begin
            // We have a single filter
            idx := cb.ItemIndex;
            if idx = -1 then continue;
            FilterID := integer(cb.Items.Objects[idx]);
            if f2.AsInteger<>FilterID then begin
               Accept := False;
               exit;
            end;
         end;
      end;

   end;
end;

procedure TDataWinForm.FastExcel971Click(Sender: TObject);
begin
   ExportTableToExcel(CurTable, False);
end;

procedure TDataWinForm.SlowExcel951Click(Sender: TObject);
begin
   ExportTableToExcel(CurTable, True);
end;

procedure TDataWinForm.ExporttoAscii1Click(Sender: TObject);
var
   s : TStringList;
begin
   if SaveDialog1.Execute then begin
      s := TStringList.Create;
      try
         ExportTableToStringList(CurTable, s);
         s.SaveToFile(SaveDialog1.FileName);
      finally
         s.Free;
      end;
   end;
end;

procedure TDataWinForm.RefreshTable1Click(Sender: TObject);
begin
   CurTable.Refresh;
   InitIndexCombo;
   StatPanel2.Caption := '  Table Refreshed';
end;

procedure TDataWinForm.ImportTablefromAscii1Click(Sender: TObject);
var
   s : TStringList;
begin
   if MessageDlg('Importing data may OVERWRITE existing data!'#13#10+
      'Do you want to continue?', mtWarning, mbYesNoCancel, 0)<>mrYes then exit;

   s := TStringList.Create;
   try
      if OpenDialog1.Execute then begin
         s.LoadFromFile(OpenDialog1.FileName);
         ImportTableFromStringList(CurTable, s, nil);
      end;
   finally
      s.Free;
   end;
end;

procedure TDataWinForm.ViewTableStructure1Click(Sender: TObject);
var
   tsf : TTableStructureForm;
begin
   tsf := TTableStructureForm.Create(Self);
   tsf.Table := CurTable;
   tsf.ShowModal;
   tsf.Free; 
end;

procedure TDataWinForm.EmptyTable1Click(Sender: TObject);
var
   rc : integer;
begin
   rc := MessageDlg('Are you SURE you want to empty the Table?',
      mtWarning, mbYesNoCancel, 0);
   if rc<>mrYes then exit;

   CurTable.Active := False;
   CurTable.Exclusive := True;

   CurTable.EmptyTable;

   CurTable.Active := False;
   CurTable.Exclusive := False;
   CurTable.Active := True;
end;

procedure TDataWinForm.TablePopupMenuPopup(Sender: TObject);
begin
   ViewStatusBar1.Checked := StatusPanel.Visible;
   Exclusive1.Checked := CurTable.Exclusive;
end;

procedure TDataWinForm.ViewStatusBar1Click(Sender: TObject);
begin
   StatusPanel.Visible := not StatusPanel.Visible;
end;

procedure TDataWinForm.Font1Click(Sender: TObject);
begin
   FontDialog1.Font := DBGrid1.Font;
   if FontDialog1.Execute then
      DBGrid1.Font := FontDialog1.Font;
end;

function TDataWinForm.SetExclusiveMode(Mode: boolean): boolean;
begin
   Result := True;
   try
      CurTable.Active := False;
      CurTable.Exclusive := Mode;
      CurTable.Active := True;
   except
      // If there is an error try to reopen without exclusive mode
      on e: Exception do begin
         MessageDlg('Exclusive Access Failed!'#13#10+e.Message, mtError, [mbOk], 0);
         CurTable.Active := False;
         CurTable.Exclusive := False;
         CurTable.Active := True;
         Result := False;
      end;
   end;
end;


procedure TDataWinForm.Exclusive1Click(Sender: TObject);
begin
   Exclusive1.Checked := not Exclusive1.Checked;
   SetExclusiveMode(Exclusive1.Checked);
end;

procedure TDataWinForm.CreateIndex1Click(Sender: TObject);
var
   cif      : TCreateIndexForm;
   OldExcl  : boolean;
begin
   // Get Exclusive Mode
   OldExcl := CurTable.Exclusive;
   if not CurTable.Exclusive then
      if not SetExclusiveMode(True) then begin
         MessageDlg('Failed to get Exclusive Mode.'#13#10'Unable to Create Index',
            mtError, [mbOK], 0);
         exit;
      end;

   // Create the Index
   cif := TCreateIndexForm.Create(Self);
   cif.Table := CurTable;
   if cif.ShowModal = mrOK then begin
      InitIndexCombo;
   end;
   cif.Free;

   if not OldExcl then
      SetExclusiveMode(False);
end;

procedure TDataWinForm.FormHint(Sender: TObject);
begin
   StatPanel2.Caption := '  ' + Application.Hint;
end;

procedure TDataWinForm.DeleteIndex1Click(Sender: TObject);
var
   i        : integer;
   OldExcl  : boolean;
   IdxName  : string;
begin
   if MessageDlg('Are you SURE you want to delete the current Index?',
      mtWarning, mbYesNoCancel, 0)<>mrYes then exit;

   // Get Exclusive Mode
   OldExcl := CurTable.Exclusive;
   if not CurTable.Exclusive then
      if not SetExclusiveMode(True) then begin
         MessageDlg('Failed to get Exclusive Mode.'#13#10'Unable to Delete Index',
            mtError, [mbOK], 0);
         exit;
      end;

   // Find the index name by fields
   CurTable.IndexDefs.Update;
   IdxName := '';
   for i := 0 to CurTable.IndexDefs.Count-1 do begin
      if CurTable.IndexDefs.Items[i].Fields = CurTable.IndexFieldNames then
         IdxName := CurTable.IndexDefs.Items[i].Name;
   end;

   // Delete the index
   if IdxName<>'' then begin
      CurTable.IndexFieldNames := '';
      CurTable.DeleteIndex(IdxName);
      CurTable.IndexDefs.Update;
      CurTable.Refresh;
      InitIndexCombo;
   end;

   if not OldExcl then
      SetExclusiveMode(False);
end;

function TFilterCombo.ShowMultForm: integer;
begin
   if MultForm=nil then begin
      MultForm := TMultFiltForm.Create(Self);
      MultForm.ListBox1.Items.Assign(Items);
   end;
   Result := MultForm.ShowModal;
   if Result = mrOK then begin
      Multiples := True;
   end;
end;

procedure TDataWinForm.MultFilt1Click(Sender: TObject);
var
   cb : TFilterCombo;
   i  : integer;
   t  : TStringList;
begin
   cb := LookupFilterPopup.PopupComponent as TFilterCombo;

   if cb.ShowMultForm = mrOK then begin
      cb.Style := csDropDown;
      // cb.Text := 'MULTIPLE: ';
      t := TStringList.Create;
      with cb.MultForm.ListBox1 do
         for i := 0 to Items.Count-1 do
            if Selected[i] then
               t.Add(Items[i]);
      cb.Text := t.CommaText;
      t.Free;

      CurTable.Refresh;
      UpdateNumRecords;
   end;
end;

function TDataWinForm.GetFilterDesc: TStringList;
var
   sl : TStringList;
   s  : string;
   i  : integer;
   fc : TControl;
   fd : TFilterDate;
   cb : TFilterCombo;
   ne : TNumFilter;
begin
   sl := TStringList.Create;
   Result := sl;

   for i := 0 to NumFilter-1 do begin
      fc := FilterPanel.FindComponent(FCNAME+IntToStr(i)) as TControl;

      if fc is TFilterDate then begin
         fd := fc as TFilterDate;

         if fd.UseMin or fd.UseMax then begin
            s := Format('%s: %s', [fd.DataField.DisplayName, fd.Caption]);
            sl.Add(s);
         end;
      end;

      if fc is TNumFilter then begin
         ne := fc as TNumFilter;
         s := Format('%s: %s', [ne.DataField.DisplayName, ne.Text]);
         sl.Add(s);
      end;

      if fc is TFilterCombo then begin
         cb := fc as TFilterCombo;

         if (cb.ItemIndex<>-1) or (cb.Multiples) then begin
            s := Format('%s: %s', [cb.DataField.DisplayName, cb.Text]);
            sl.Add(s);
         end;
      end;
   end;
end;

procedure TDataWinForm.ResetTableViewtoDefault1Click(Sender: TObject);
var
   i        : integer;
   DefCols  : TMemoryStream;
begin
   i := DefColList.IndexOf(CurTable.TableName);
   if i=-1 then exit;
   DefCols := TMemoryStream(DefColList.Objects[i]);

   with DBGrid1.Columns do begin
      DefCols.Position := 0;
      LoadFromStream(DefCols);
   end;
end;

function TDataWinForm.GetColSetName: string;
begin
   Result := ExtractFilePath(Application.ExeName) + '\' + CurTable.TableName +
      '.col';
end;

procedure TDataWinForm.SaveColumnSettings;
var
   fs : TFileStream;
begin
   if CurTable=nil then exit;

   fs := TFileStream.Create(GetColSetName, fmCreate or fmOpenWrite or fmShareExclusive);
   DBGrid1.Columns.SaveToStream(fs);
   fs.Free;
end;

procedure TDataWinForm.RestoreColumnSettings;
var
   fn  : string;
   fs  : TFileStream;
begin
   fn := GetColSetName;

   if FileExists(fn) then begin
      fs := TFileStream.Create(GetColSetName, fmOpenRead or fmShareExclusive);
      DBGrid1.Columns.LoadFromStream(fs);
      fs.Free;
   end;
end;

procedure TDataWinForm.SaveDefaultColumnSettings;
var
   i        : integer;
   DefCols  : TMemoryStream;
begin
   i := DefColList.IndexOf(CurTable.TableName);
   if i<>-1 then exit;

   DefCols := TMemoryStream.Create;

   with DBGrid1.Columns do begin
      State := csCustomized;
      RebuildColumns;
      // Get rid of invisible columns
      for i := Count-1 downto 0 do
         if not Items[i].Field.Visible then
            Items[i].Free;
      SaveToStream(DefCols);
   end;

   DefColList.AddObject(CurTable.TableName, DefCols);
end;


procedure TDataWinForm.NumLabelClick(Sender: TObject);
var
   ne : TNumFilter;
begin
   ne := (Sender as TLabel).FocusControl as TNumFilter;
   ne.UseMin := False;
   ne.UseMax := False;
   ne.IncBlanks := True;
   ne.UpdateText;

   CurTable.Refresh;
   UpdateNumRecords;
end;

procedure TDataWinForm.NumKeyPress(Sender: TObject; var Key: char);
var
   ne : TNumFilter;
begin
   ne := Sender as TNumFilter;

   if Key = #13 then begin
      ne.UseOneVal;

      ne.UpdateText;
      Key := #0;
      CurTable.Refresh;
      UpdateNumRecords;
   end;
end;


procedure TDataWinForm.FilteronRange1Click(Sender: TObject);
var
   ne : TNumFilter;
begin
   ne := NumFilterPopup.PopupComponent as TNumFilter;

   if NumFilterForm=nil then
      NumFilterForm := TNumFilterForm.Create(Self);

   NumFilterForm.ne := ne;
   if NumFilterForm.ShowModal=mrOK then begin
      CurTable.Refresh;
      UpdateNumRecords;
   end;
end;

end.
